# Generated by Django 2.0.3 on 2018-03-23 21:37

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

	dependencies = [
		("djpaypal", "0003_auto_20180320_0902"),
	]

	operations = [
		migrations.RenameField(
			model_name="dispute", old_name="id", new_name="dispute_id",
		),
		migrations.AlterField(
			model_name="billingagreement",
			name="override_charge_mode",
			field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict),
		),
		migrations.AlterField(
			model_name="billingagreement",
			name="override_merchant_preferences",
			field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict),
		),
	]
