{% extends "admin/change_form.html" %}
{% load i18n %}

{% block object-tools-items %}
	{{ block.super }}
	{% with original.dashboard_url as dashboard_url %}
	{% if dashboard_url %}
		<li><a href="{{ dashboard_url }}" target="_blank" id="djpaypal-view-on-paypal"><i class="icon-user icon-alpha75"></i>{% trans "View on Paypal Dashboard" %}</a></li>
	{% endif %}
	{% endwith %}

	{% with original.refund_url as refund_url %}
	{% if refund_url %}
		<li><a href="{{ refund_url }}" target="_blank" id="djpaypal-issue-refund"><i class="icon-user icon-alpha75"></i>{% trans "Issue refund" %}</a></li>
	{% endif %}
	{% endwith %}
{% endblock %}
